import logging
import os.path
import threading

from flask import Flask, request

from wi1_bot import push, transcoder
from wi1_bot.arr import Radarr, Sonarr
from wi1_bot.config import config

app = Flask(__name__)

logging.getLogger("werkzeug").disabled = True

logger = logging.getLogger(__name__)

radarr = Radarr(config["radarr"]["url"], config["radarr"]["api_key"])
sonarr = Sonarr(config["sonarr"]["url"], config["sonarr"]["api_key"])


def on_grab(req: dict) -> None:
    push.send(
        req["release"]["releaseTitle"], title=f"file grabbed ({req['downloadClient']})"
    )


def on_download(req: dict) -> None:
    if "movie" in req:
        content_id = req["movie"]["id"]

        quality_profile = radarr.get_quality_profile_name(
            radarr._radarr.get_movie_by_movie_id(content_id)["qualityProfileId"]
        )

        movie_folder = req["movie"]["folderPath"]
        basename = req["movieFile"]["relativePath"]

        push.send(basename, title="movie downloaded")

        path = os.path.join(movie_folder, basename)
    elif "series" in req:
        content_id = req["series"]["id"]

        quality_profile = sonarr.get_quality_profile_name(
            sonarr._sonarr.get_series(content_id)["qualityProfileId"]
        )

        series_folder = req["series"]["path"]
        relative_path = req["episodeFile"]["relativePath"]

        push.send(relative_path.split("/")[-1], title="episode downloaded")

        path = os.path.join(series_folder, relative_path)
    else:
        raise ValueError("unknown download request")

    try:
        quality_options = config["transcoding"]["profiles"][quality_profile]
    except KeyError:
        return

    def get_key(d, k):
        try:
            return d[k]
        except KeyError:
            return None

    video_codec = get_key(quality_options, "video_codec")
    video_bitrate = get_key(quality_options, "video_bitrate")
    audio_codec = get_key(quality_options, "audio_codec")
    audio_channels = get_key(quality_options, "audio_channels")
    audio_bitrate = get_key(quality_options, "audio_bitrate")

    transcoder.queue.add(
        path=path,
        video_codec=video_codec,
        video_bitrate=video_bitrate,
        audio_codec=audio_codec,
        audio_channels=audio_channels,
        audio_bitrate=audio_bitrate,
        content_id=content_id,
    )


@app.route("/", methods=["POST"])
def index():
    try:
        if request.json is None or "eventType" not in request.json:
            return "", 400

        logger.debug(f"got request: {request.json}")

        if request.json["eventType"] == "Grab":
            on_grab(request.json)
        elif request.json["eventType"] == "Download":
            on_download(request.json)
    except Exception:
        logger.warning(f"error handling request: {request.data}", exc_info=True)

    return "", 200


def start() -> None:
    logger.debug("starting webhook listener")

    t = threading.Thread(target=app.run, kwargs={"host": "localhost", "port": 9000})
    t.daemon = True
    t.start()


if __name__ == "__main__":
    app.run(host="localhost", port=9000)



import logging
import logging.config

from wi1_bot import webhook
from wi1_bot.discord import bot
from wi1_bot.transcoder import Transcoder


def main():
    logging_config = {
        "version": 1,
        "disable_existing_loggers": True,
        "formatters": {
            "basic": {
                "class": "logging.Formatter",
                "format": "[%(asctime)s] %(levelname)s %(message)s",
                "datefmt": "%Y-%m-%d %H:%M:%S",
            },
            "detailed": {
                "class": "logging.Formatter",
                "format": (
                    "[%(asctime)s] %(levelname)s [%(name)s.%(funcName)s:%(lineno)d]"
                    " %(message)s"
                ),
            },
        },
        "handlers": {
            "console": {
                "class": "logging.StreamHandler",
                "stream": "ext://sys.stdout",
                "level": "DEBUG",
                "formatter": "detailed",
            },
            # "file": {
            #     "class": "logging.handlers.RotatingFileHandler",
            #     "filename": "logs/wi1-bot.log",
            #     "maxBytes": 1024**2 * 10,  # 10 MB
            #     "backupCount": 20,
            #     "level": "INFO",
            #     "formatter": "basic",
            # },
            # "file_debug": {
            #     "class": "logging.handlers.RotatingFileHandler",
            #     "filename": "logs/wi1-bot.debug.log",
            #     "maxBytes": 1024**2 * 10,  # 10 MB
            #     "backupCount": 20,
            #     "level": "DEBUG",
            #     "formatter": "detailed",
            # },
        },
        "loggers": {
            "": {"level": "DEBUG", "handlers": ["console"]},
            "wi1_bot": {"level": "DEBUG", "handlers": [], "propagate": True},
        },
    }

    logging.config.dictConfig(logging_config)

    webhook.start()

    t = Transcoder(ws=True)
    t.start()

    try:
        bot.run()
    except KeyboardInterrupt:
        pass


if __name__ == "__main__":
    main()



import os
from typing import TypedDict

import yaml

_config_path: str | None = None

if os.path.isfile("config.yaml"):
    _config_path = "config.yaml"

if dir := os.getenv("XDG_CONFIG_HOME"):
    if os.path.isfile(os.path.join(dir, "wi1-bot", "config.yaml")):
        _config_path = os.path.join(dir, "wi1-bot", "config.yaml")

if home := os.getenv("HOME"):
    if os.path.isfile(os.path.join(home, ".config", "wi1-bot", "config.yaml")):
        _config_path = os.path.join(home, ".config", "wi1-bot", "config.yaml")

if _config_path is None:
    raise FileNotFoundError(
        "could not find ./config.yaml, $XDG_CONFIG_HOME/wi1-bot/config.yaml or"
        " $HOME/.config/wi1-bot/config.yaml"
    )


class ArrConfig(TypedDict):
    url: str
    api_key: str


class PushoverConfig(TypedDict):
    user_key: str
    api_key: str
    devices: str


# FIXME: python 3.11 supports typing.NotRequired (PEP 655), so we
# won't have to deal with this horrid mess of
# ___ConfigRequired followed by ___Config(___ConfigRequired, total=False)
class DiscordConfigRequired(TypedDict):
    bot_token: str
    channel_id: int
    admin_id: int


class DiscordConfig(DiscordConfigRequired, total=False):
    bot_presence: str  # optional
    quotas: dict[int, float]  # optional


class TranscodingProfile(TypedDict, total=False):
    video_codec: str  # optional
    video_bitrate: int  # optional
    audio_codec: str  # optional
    audio_channels: int  # optional
    audio_bitrate: str  # optional


class TranscodingConfigRequired(TypedDict):
    profiles: dict[str, TranscodingProfile]


class TranscodingConfig(TranscodingConfigRequired, total=False):
    hwaccel: str  # optional


class ConfigRequired(TypedDict):
    radarr: ArrConfig
    sonarr: ArrConfig
    discord: DiscordConfig


class Config(ConfigRequired, total=False):
    pushover: PushoverConfig  # optional
    transcoding: TranscodingConfig  # optional


with open(_config_path, "r") as f:
    config: Config = yaml.load(f, Loader=yaml.SafeLoader)



class Movie:
    def __init__(self, movie_json: dict) -> None:
        self._json = movie_json

        self.title: str = movie_json["title"]
        self.year: int = movie_json["year"]
        self.tmdb_id: int = movie_json["tmdbId"]

        self.full_title = f"{self.title} ({self.year})"

        self.url = f"https://themoviedb.org/movie/{self.tmdb_id}"

        self.imdb_id: str = ""

        try:
            self.imdb_id = movie_json["imdbId"]
            self.url = f"https://imdb.com/title/{self.imdb_id}"
        except KeyError:
            pass

    def __str__(self) -> str:
        return f"[{self.full_title}]({self.url})"

    def __repr__(self) -> str:
        return str(self.__dict__)



import argparse

from wi1_bot.config import config
from wi1_bot.transcoder import queue


def main():
    parser = argparse.ArgumentParser(description="add item to transcode queue")

    parser.add_argument("path", help="file path to transcode")

    args = parser.parse_args()

    qp = config["transcoding"]["profiles"]["good"]

    queue.add(
        path=args.path,
        video_codec=qp["video_codec"],
        video_bitrate=qp["video_bitrate"],
        audio_codec=qp["audio_codec"],
        audio_channels=qp["audio_channels"],
        audio_bitrate=qp["audio_bitrate"],
    )

    print("place in queue:", queue.size)


if __name__ == "__main__":
    main()
